@@ -8,9 +8,17 @@ test -n "$LMDBG_LIB"
 LC_ALL=C
 export LC_ALL
 
+: ${LMDBG_PROCFS:=/proc}
+if test -f "$LMDBG_PROCFS/$$/maps"; then
+    procfs_exists=1
+fi
+
 unify_paths (){
     # /home/cheusov/prjs/lmdbg/ ---> /lmdbg/dir/
-    sed 's,/[^ ]*lmdbg[^ ]*/,/lmdbg/dir/,g' "$@"
+    sed -e 's,\([^[:space:]]*\)/prog,/path/to/prog,' \
+	-e 's,'`dirname "$LMDBG_LIB"`',/lmdbg/dir,g' \
+	-e 's,/[^ ]*lmdbg[^ ]*/,/lmdbg/dir/,g' \
+	-e 's,[^[:space:]]*/\([^ /]*[.]c\),\1,' "$@"
 }
 
 unify_paths_inplace (){
@@ -26,7 +34,7 @@ unify_address (){
     awk '
 	$1 == "malloc"         {$6 = "0xF00DBEAF"}
 	$1 == "calloc"         {$8 = "0xF00DBEAF"}
-	$1 == "posix_memalign" && $8 != "NULL" {$8 = "0xF00DBEAF"}
+	($1 == "posix_memalign" || $1 == "aligned_alloc") && $8 != "NULL" {$8 = "0xF00DBEAF"}
 	$1 == "free"           {$3 = "0xF00DBEAF"}
 	$1 == "realloc"        {$8 = "0xF00DBEAF"}
 	$1 == "realloc" && $3 != "NULL" {$3 = "0xF00DBEAF"}
@@ -39,10 +47,22 @@ skip_info (){
     grep -v '^info ' "$@"
 }
 
+skip_foreign_mallocs (){
+    lmdbg-grep '!/malloc/ || /[(] (1|2|555|666|777|888) [)]/' "$@"
+}
+
 skip_useless_addr (){
     awk 'NF != 1 || $1 != "0xF00DBEAF"' "$@"
 }
 
+hide_num (){
+    sed 's/num: [0-9][0-9]*/num: MMM/'
+}
+
+skip_all (){
+    skip_foreign_mallocs "$@" | skip_useless_addr | hide_num
+}
+
 hide_lmdbg_code (){
     # cut off line like the following
     #   0xF00DBEAF  /GNU/libc/code/source.c     some_func
@@ -64,9 +84,9 @@ version2XXX (){
 }
 
 hide_foreign_code (){
-    awk -F'\t' '
+    awk -F'\t' -v procfs_exists="$procfs_exists" '
 	{ sub(/wrap_/, "") }
-	$0 ~ /^[^ ]/ || $2 ~ /lmdbg[.]c|prog.[.]c/
+	$0 ~ /^[^ ]/ || /prog.[.]c|test3.c/ || (procfs_exists && $2 ~ /lmdbg[.]c/)
     ' "$@"
 }
 
@@ -126,197 +146,391 @@ cmp "lmdbg-grep -h" \
 'Taking an output of lmdbg-stat on input lmdbg-grep outputs global
 '
 
-lmdbg-run -V               | head -1 | version2XXX |
+lmdbg-head -h | head -1 |
+cmp "lmdbg-head -h" \
+'Taking an output of lmdbg on input, lmdbg-head outputs global
+'
+
+lmdbg-run -V | head -1 | version2XXX |
 cmp "lmdbg-run -V" \
 "lmdbg-run XXX
 "
 
-lmdbg-sym -V               | head -1 | version2XXX |
+lmdbg-sym -V | head -1 | version2XXX |
 cmp "lmdbg-sym -V" \
 "lmdbg-sym XXX
 "
 
-lmdbg-leaks -V             | head -1 | version2XXX |
+lmdbg-leaks -V | head -1 | version2XXX |
 cmp "lmdbg-leaks -V" \
 "lmdbg-leaks XXX
 "
 
-lmdbg-sysleaks -V          | head -1 | version2XXX |
+lmdbg-sysleaks -V | head -1 | version2XXX |
 cmp "lmdbg-sysleaks -V" \
 "lmdbg-sysleaks XXX
 "
 
-lmdbg-stat -V              | head -1 | version2XXX |
+lmdbg-stat -V | head -1 | version2XXX |
 cmp 'lmdbg-stat -V' \
 'lmdbg-stat XXX
 '
 
-lmdbg-grep -V              | head -1 | version2XXX |
+lmdbg-grep -V | head -1 | version2XXX |
 cmp 'lmdbg-grep -V' \
 'lmdbg-grep XXX
 '
 
-lmdbg-modules -V       | head -1 | version2XXX |
+lmdbg-modules -V | head -1 | version2XXX |
 cmp "lmdbg-modules -V" \
 "lmdbg-modules XXX
 "
 
-lmdbg-strip -V       | head -1 | version2XXX |
+lmdbg-strip -V | head -1 | version2XXX |
 cmp "lmdbg-strip -V" \
 "lmdbg-strip XXX
 "
 
+lmdbg-head -V | head -1 | version2XXX |
+cmp "lmdbg-head -V" \
+"lmdbg-head XXX
+"
+
 ####################
 # normal tests
 
-# test1.c
-if test -d "$OBJDIR/test"; then
-    execname1="$OBJDIR"/test/prog1/prog1
-    execname2="$OBJDIR"/test/prog2/prog2
-    execname4="$OBJDIR"/test/prog4/prog4
-    execname5="$OBJDIR"/test/prog5/prog5
-    execname6="$OBJDIR"/test/prog6/prog6
-
-    logname="$OBJDIR"/_log
-
-    libname="$OBJDIR"/libtest3/libtest3.so
-
-    exec3name="$OBJDIR"/test/prog3/prog3
-
-    LD_LIBRARY_PATH=$OBJDIR/test/libtest3
-else
-    execname1="$OBJDIR"/prog1
-    execname2="$OBJDIR"/prog2
-    execname4="$OBJDIR"/prog4
-    execname5="$OBJDIR"/prog5
-    execname6="$OBJDIR"/prog6
-
-    logname="$OBJDIR"/_log
-
-    libname="$OBJDIR"/libtest3.so
-
-    exec3name="$OBJDIR"/prog3
-
-    LD_LIBRARY_PATH=$OBJDIR
-fi
-
 #
-export LD_LIBRARY_PATH
+execname1=`which prog1 || true`
+execname2=`which prog2 || true`
+execname3=`which prog3 || true`
+execname4=`which prog4 || true`
+execname5=`which prog5 || true`
+execname6=`which prog6 || true`
+execname7=`which prog7 || true`
+execname8=`which prog8 || true`
+execname9=`which prog9 || true`
+logname="$OBJDIR"/_log
+pidfile="$OBJDIR"/_pid
+
+# lmdbg-run and $#
+if lmdbg-run -o "$logname" "$execname1 1 2 3 4" 2>/dev/null; then
+    echo FAILED
+else
+    echo ok
+fi |
+cmp 'lmdbg-run and $# #1' \
+'ok
+'
+
+if ! lmdbg-run -o "$logname" "$execname1" '1 1 1' 2 '3 3' 4 2>/dev/null; then
+    echo FAILED
+fi |
+cmp 'lmdbg-run and $# #2' \
+'argc=4
+'
+
+# lmdbg-run -o with no progname
+if lmdbg-run -o "$logname" 2>/dev/null; then
+    echo FAILED
+else
+    echo ok
+fi |
+cmp 'lmdbg-run -o with no progname' \
+'ok
+'
+
+# lmdbg-run -o with empty progname
+if lmdbg-run -o "$logname" '    ' 2>/dev/null; then
+    echo FAILED
+else
+    echo ok
+fi |
+cmp 'lmdbg-run -o with empty progname' \
+'ok
+'
+
+if lmdbg-run -v -o "$logname" "$execname5" 2>&1; then
+    echo FAILED
+else
+    echo $?
+fi |
+unify_paths |
+cmp 'lmdbg-run: test for exit status' \
+'Running /path/to/prog5 with LD_PRELOAD=/lmdbg/dir/liblmdbg.so
+Done.
+17
+'
+
+# lmdbg-leaks
+cat > "$logname" <<EOF
+info section 0x00000000400000 0x00000000401000
+info section 0x007f7ff7400000 0x007f7ff751f000
+info section 0x007f7ff7c00000 0x007f7ff7c12000
+info progname ./hello_SLIST
+malloc ( 16384 ) --> 0x7f7ff7b01000 num: 1
+ 0x7f7ff74f0a8a
+malloc ( 10 ) --> 0x7f7ff7b09080 num: 2
+ 0x7f7ff74dc6e7
+calloc ( 1 , 16 ) --> 0x7f7ff7b09090 num: 3
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 10 ) --> 0x7f7ff7b090a0 num: 4
+ 0x7f7ff74dc6e7
+calloc ( 1 , 16 ) --> 0x7f7ff7b090b0 num: 5
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 11 ) --> 0x7f7ff7b090c0 num: 6
+ 0x7f7ff74dc6e7
+calloc ( 1 , 16 ) --> 0x7f7ff7b090d0 num: 7
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 4096 ) --> 0x7f7ff7b0b000 num: 8
+ 0x7f7ff74f0a8a
+free ( 0x7f7ff7b090c0 ) num: 9
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090d0 ) num: 10
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090a0 ) num: 11
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090b0 ) num: 12
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b09080 ) num: 13
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b09090 ) num: 14
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+EOF
+
+lmdbg-leaks < "$logname" |
+cmp 'lmdbg-leaks real #1' \
+'info progname ./hello_SLIST
+info section 0x00000000400000 0x00000000401000
+info section 0x007f7ff7400000 0x007f7ff751f000
+info section 0x007f7ff7c00000 0x007f7ff7c12000
+malloc ( 16384 ) --> 0x7f7ff7b01000 num: 1
+ 0x7f7ff74f0a8a
+malloc ( 4096 ) --> 0x7f7ff7b0b000 num: 8
+ 0x7f7ff74f0a8a
+'
+
+cat > "$logname" <<EOF
+info progname ./hello_SLIST
+info section 0x00000000400000 0x00000000401000
+info section 0x007f7ff7400000 0x007f7ff751f000
+info section 0x007f7ff7c00000 0x007f7ff7c12000
+malloc ( 16384 ) --> 0x7f7ff7b01000 num: 1
+ 0x7f7ff74f0a8a
+malloc ( 10 ) --> 0x7f7ff7b09080 num: 2
+ 0x7f7ff74dc6e7
+malloc ( 16 ) --> 0x7f7ff7b09090 num: 3
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 10 ) --> 0x7f7ff7b090a0 num: 4
+ 0x7f7ff74dc6e7
+malloc ( 16 ) --> 0x7f7ff7b090b0 num: 5
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 11 ) --> 0x7f7ff7b090c0 num: 6
+ 0x7f7ff74dc6e7
+malloc ( 16 ) --> 0x7f7ff7b090d0 num: 7
+ 0x400c68
+ 0x400d0e
+ 0x400b13
+malloc ( 4096 ) --> 0x7f7ff7b0b000 num: 8
+ 0x7f7ff74f0a8a
+free ( 0x7f7ff7b090c0 ) num: 9
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090d0 ) num: 10
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090a0 ) num: 11
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b090b0 ) num: 12
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b09080 ) num: 13
+ 0x400c22
+ 0x400d33
+ 0x400b13
+free ( 0x7f7ff7b09090 ) num: 14
+ 0x400c3f
+ 0x400d33
+ 0x400b13
+EOF
+
+lmdbg-leaks < "$logname" |
+cmp 'lmdbg-leaks real #2' \
+'info progname ./hello_SLIST
+info section 0x00000000400000 0x00000000401000
+info section 0x007f7ff7400000 0x007f7ff751f000
+info section 0x007f7ff7c00000 0x007f7ff7c12000
+malloc ( 16384 ) --> 0x7f7ff7b01000 num: 1
+ 0x7f7ff74f0a8a
+malloc ( 4096 ) --> 0x7f7ff7b0b000 num: 8
+ 0x7f7ff74f0a8a
+'
+
+# -o
+rm -f "$logname"
+
+if echo qwerty | lmdbg-run -o "$logname" grep qwerty > /dev/null 2>&1; then
+    echo ok
+else
+    echo $?
+fi |
+cmp 'stacktrace(3): test for sigsegv seen on NetBSD/x86_64' \
+'ok
+'
+
+if grep '^ ' "$logname" 2>&1 > /dev/null; then
+    echo ok
+else
+    echo "grep(1) exit status is $?"
+fi |
+cmp 'lmdbg-run: non-empty log' \
+'ok
+'
+
+# -o
+if lmdbg-run -o "$logname" "$execname7" 2>/dev/null 1>&2; then
+    echo FAILED
+else
+    echo $?
+fi |
+cmp 'stacktrace(3): test for foreign sigsegv' \
+'139
+'
 
 # -o
 lmdbg-run -o "$logname" "$execname1"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all | head -5 |
 cmp "prog1.c: lmdbg-run -o" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 "
 
 # -n
 lmdbg-run -o "$logname" -n "$execname1"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all |
 cmp "prog1.c: lmdbg-run -n" \
 ''
 
 # -T
 lmdbg-run -o "$logname" -T1 "$execname1"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all | head -5 |
 cmp "prog1.c: lmdbg-run -T" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 "
 
 # -B
 lmdbg-run -o "$logname" -B2 "$execname1"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all | head -5 |
 cmp "prog1.c: lmdbg-run -B" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 "
 
 # -o
 lmdbg-run -o "$logname" "$execname1"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all | head -5 |
 cmp "prog1.c: lmdbg-run -o" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 "
 
 # lmdbg-leaks
 logname2="$OBJDIR"/_log2
 lmdbg-leaks "$logname" > "$logname2"
 
-unify_address "$logname2" | skip_info | skip_useless_addr |
+unify_address "$logname2" | skip_info | skip_all |
 cmp "prog1.c: lmdbg-leaks" \
-"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
 "
 
 # lmdbg-sym -g
 lmdbg-sym -g "$logname" |
 unify_address | skip_info | hide_lmdbg_code | hide_line_numbers |
-canonize_paths | skip_useless_addr |
+canonize_paths | skip_all | lmdbg-head -n 5 |
 cmp "prog1.c: lmdbg-sym" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-free ( 0xF00DBEAF ) num: 5
+free ( 0xF00DBEAF ) num: MMM
  0xF00DBEAF	prog1.c:NNN	main
 "
 
 # lmdbg-sym -g
 lmdbg-sym -g -P "$execname1" "$logname" |
 unify_address | skip_info | hide_lmdbg_code | hide_line_numbers |
-canonize_paths | skip_useless_addr |
+canonize_paths | skip_all | lmdbg-head -n 5 |
 cmp "prog1.c: lmdbg-sym -g" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
-free ( 0xF00DBEAF ) num: 5
+free ( 0xF00DBEAF ) num: MMM
  0xF00DBEAF	prog1.c:NNN	main
 "
 
 if addr2line --help >/dev/null 2>&1 && test `uname -s` != SunOS; then
-# lmdbg-sym -a
     lmdbg-sym -a -P "$execname1" "$logname" |
     unify_address | skip_info | hide_lmdbg_code | hide_line_numbers |
-    canonize_paths | skip_useless_addr |
+    canonize_paths | skip_all |  lmdbg-head -n 5 | grep -v '0xF00DBEAF.*prog1' |
     cmp "prog1.c: lmdbg-sym -a" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
- 0xF00DBEAF	prog1.c:NNN
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
- 0xF00DBEAF	prog1.c:NNN
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
- 0xF00DBEAF	prog1.c:NNN
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
- 0xF00DBEAF	prog1.c:NNN
-free ( 0xF00DBEAF ) num: 5
- 0xF00DBEAF	prog1.c:NNN
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 "
 fi
 
@@ -325,9 +539,9 @@ lmdbg-run -o"$logname" -f lmdbg-leaks "$execname1"
 
 unify_address "$logname" | skip_info | hide_lmdbg_code |
 hide_line_numbers |
-canonize_paths | skip_useless_addr |
+canonize_paths | skip_all |
 cmp "prog1.c: lmdbg-run -f lmdbg-leaks" \
-"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
 "
 
 # lmdbg-run -f lmdbg-leaks
@@ -335,9 +549,9 @@ lmdbg-run -o"$logname" -f lmdbg-leaks "$execname1"
 
 unify_address "$logname" | skip_info | hide_lmdbg_code |
 hide_line_numbers |
-canonize_paths | skip_useless_addr |
+canonize_paths | skip_all |
 cmp "prog1.c: lmdbg-run -f lmdbg-leaks" \
-"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+"realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
 "
 
 # prog1.c
@@ -346,25 +560,17 @@ logname="$OBJDIR"/_log
 # lmdbg-run -o with two leaks
 lmdbg-run -o "$logname" -f"lmdbg-sym" "$execname2"
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all |
 hide_line_numbers |
-canonize_paths | skip_useless_addr | hide_foreign_code |
+canonize_paths | skip_all | hide_foreign_code | lmdbg-head -n 4 |
 cmp "prog2.c: lmdbg-run -f" \
-"malloc ( 555 ) --> 0xF00DBEAF num: 1
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	malloc
+"malloc ( 555 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog2.c:NNN	main
-realloc ( NULL , 666 ) --> 0xF00DBEAF num: 2
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+realloc ( NULL , 666 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog2.c:NNN	main
-realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: 3
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog2.c:NNN	main
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog2.c:NNN	main
 "
 
@@ -374,17 +580,13 @@ lmdbg-leaks "$logname" > "$logname2"
 
 unify_paths_inplace "$logname"
 
-unify_paths "$logname2" | skip_info | unify_address | skip_useless_addr |
+unify_paths "$logname2" | skip_info | unify_address | skip_all |
 hide_line_numbers | hide_foreign_code | sort |
 cmp "prog2.c: lmdbg-run -p again" \
-" 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	malloc
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+" 0xF00DBEAF	prog2.c:NNN	main
  0xF00DBEAF	prog2.c:NNN	main
- 0xF00DBEAF	prog2.c:NNN	main
-malloc ( 555 ) --> 0xF00DBEAF num: 1
-realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
+malloc ( 555 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
 "
 
 # lmdbg-leaks with lmdbg-leak1.conf
@@ -417,8 +619,8 @@ EOF
 
 lmdbg-sysleaks -c ./lmdbg1.conf -s > "$logname2" < "$logname"
 
-unify_address "$logname2" | skip_info | skip_useless_addr |
-hide_line_numbers | hide_foreign_code |
+unify_address "$logname2" | skip_info | skip_all |
+hide_line_numbers |
 cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg1.conf" \
 "malloc ( 555 ) --> 0xF00DBEAF
  0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
@@ -442,7 +644,7 @@ realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF
 lmdbg-sysleaks -c ./lmdbg2.conf -s \
     "$logname" > "$logname2"
 
-unify_address "$logname2" | skip_info | skip_useless_addr |
+unify_address "$logname2" | skip_info | skip_all |
 hide_line_numbers |
 canonize_paths |
 cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg2.conf -s" ''
@@ -451,8 +653,8 @@ cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg2.conf -s" ''
 lmdbg-sysleaks -c ./lmdbg3.conf -s \
     "$logname" > "$logname2"
 
-unify_address "$logname2" | skip_info | skip_useless_addr |
-hide_line_numbers | hide_foreign_code |
+unify_address "$logname2" | skip_info | skip_all |
+hide_line_numbers |
 cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg3.conf -s" \
 'malloc ( 555 ) --> 0xF00DBEAF
  0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
@@ -472,8 +674,8 @@ realloc ( 0xF00DBEAF , 777 ) --> 0xF00DBEAF
 lmdbg-sysleaks -c ./lmdbg3.conf \
     "$logname" > "$logname2"
 
-unify_address "$logname2" | skip_info | skip_useless_addr |
-hide_line_numbers | hide_foreign_code |
+unify_address "$logname2" | skip_info | skip_all |
+hide_line_numbers |
 cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg3.conf" \
 'realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF
  0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
@@ -484,42 +686,42 @@ cmp "prog2.c: lmdbg-sysleaks -c ./lmdbg3.conf" \
 # lmdbg!
 lmdbg -c ./lmdbg3.conf -o "$logname" "$execname1" || true
 
-unify_paths "$logname" | skip_useless_addr |
+unify_paths "$logname" | skip_all |
 hide_line_numbers | unify_address | skip_info | hide_foreign_code |
 cmp "prog1.c: lmdbg -c ./lmdbg3.conf" \
-'realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: 4
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+'realloc ( 0xF00DBEAF , 888 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog1.c:NNN	main
 '
 
 # lmdbg!
+if false; then
+    # disabled because NetBSD-9.0 allocates memory
+    # somewhere in libc and does not free it.
 lmdbg -v -c ./lmdbg5.conf -o "$logname" "$execname1" 2>"$logname2" || true
 
 cat "$logname2" |
 cmp "prog1.c: lmdbg -v -c lmdbg5.conf" \
 'No memory leaks detected
 '
+fi
 
 # lmdbg!
 lmdbg -c ./lmdbg6.conf -o "$logname" "$execname2" || true
 
-unify_address "$logname" | skip_info | skip_useless_addr |
+unify_address "$logname" | skip_info | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code |
 cmp "prog1.c: lmdbg -o" \
-'malloc ( 555 ) --> 0xF00DBEAF num: 1
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	malloc
+'malloc ( 555 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog2.c:NNN	main
 '
 
 # lmdbg -M allocs!
 lmdbg -T2 -M allocs -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M allocs #1" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
@@ -527,17 +729,17 @@ stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 '
 
 # lmdbg -M allocs!
 lmdbg -T2 -Ma -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -Ma #2" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
@@ -545,23 +747,23 @@ stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 '
 
 # lmdbg -M peak!
 lmdbg -T2 -Mpeak -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M peak #1" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
 stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
  	prog6.c:NNN	main
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
@@ -570,16 +772,16 @@ stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
 # lmdbg -M p!
 lmdbg -T2 -M p -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M p #2" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
 stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
  	prog6.c:NNN	main
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
@@ -588,14 +790,14 @@ stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
 # lmdbg -M max!
 lmdbg -T2 -Mmax -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M max #1" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
@@ -606,14 +808,14 @@ stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
 # lmdbg -M m!
 lmdbg -T2 -M m -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M m #2" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2201
 info stat total_free_cnt: 1
 info stat total_leaks: 2400
-stacktrace peak: 1000 max: 1000 allocs: 1
+stacktrace peak: 888 max: 888 allocs: 1
  	prog6.c:NNN	main
 stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
@@ -624,10 +826,10 @@ stacktrace peak: 2000 max: 1 allocs: 2000 leaks: 2000
 # lmdbg -M leaks!
 lmdbg -T2 -M leaks -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M leaks #1" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2200
 info stat total_free_cnt: 0
 info stat total_leaks: 2400
@@ -640,10 +842,10 @@ stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
 # lmdbg -M leaks!
 lmdbg -T2 -M l -o "$logname" "$execname6" || true
 
-unify_address "$logname" | skip_useless_addr |
+unify_address "$logname" | skip_all |
 hide_line_numbers | unify_paths | hide_foreign_code | grep -E '^[^ ]|main' |
 cmp "prog6.c: lmdbg -M l #2" \
-'info progname /lmdbg/dir/prog6
+'info progname /path/to/prog6
 info stat total_allocs: 2200
 info stat total_free_cnt: 0
 info stat total_leaks: 2400
@@ -653,25 +855,37 @@ stacktrace peak: 400 max: 2 allocs: 200 leaks: 400
  	prog6.c:NNN	main
 '
 
-# lmdbg-run -o and shared libraries
-lmdbg-run -o "$logname" "$exec3name"
+# lmdbg-run -nN
+lmdbg-run -o "$logname" -nN "$pidfile" "$execname8" &
+sleep 1
+kill -USR1 `cat $pidfile`
+wait
 
-unify_address "$logname" | skip_info | skip_useless_addr |
-hide_line_numbers | unify_paths |
+unify_address "$logname" | skip_info | skip_all | grep -E ' (555|666) ' |
+cmp "prog8.c: lmdbg-run + SIGUSR1" \
+"malloc ( 666 ) --> 0xF00DBEAF num: MMM
+"
+
+# lmdbg-run -o and shared libraries
+lmdbg-run -o "$logname" "$execname3"
+
+unify_address "$logname" | skip_info | skip_all |
+    grep -E ' (555|666) ' |
+    hide_line_numbers | unify_paths |
 cmp "prog3.c: lmdbg-run -o" \
-'malloc ( 555 ) --> 0xF00DBEAF num: 1
-malloc ( 666 ) --> 0xF00DBEAF num: 2
+'malloc ( 555 ) --> 0xF00DBEAF num: MMM
+malloc ( 666 ) --> 0xF00DBEAF num: MMM
 '
 
 # lmdbg-sym -g and shared libraries
-lmdbg-sym -g -P "$exec3name" "$logname" |
-unify_paths | unify_address | skip_info | hide_lmdbg_code | hide_line_numbers |
-skip_useless_addr | 
+lmdbg-sym -g -P "$execname3" "$logname" |
+    unify_paths | unify_address | skip_info | hide_lmdbg_code | skip_foreign_mallocs |
+    hide_line_numbers | lmdbg-head -n 2 | hide_num | skip_useless_addr |
 cmp "prog3.c: lmdbg-sym -g" \
-'malloc ( 555 ) --> 0xF00DBEAF num: 1
+'malloc ( 555 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	test3.c:NNN	allocate_memory
  0xF00DBEAF	prog3.c:NNN	main
-malloc ( 666 ) --> 0xF00DBEAF num: 2
+malloc ( 666 ) --> 0xF00DBEAF num: MMM
  0xF00DBEAF	prog3.c:NNN	main
 '
 
@@ -682,21 +896,21 @@ if test "$with_glibc" = 0; then
     lmdbg-run -o "$logname" "$execname4"
 
     unify_address "$logname" | skip_info | 
-    skip_useless_addr |
+    skip_all | head -5 |
     cmp "prog4.c: lmdbg-run -o" \
-'calloc ( 555 , 16 ) --> 0xF00DBEAF num: 1
-calloc ( 5 , 256 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: 3
-calloc ( 1 , 10240 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+'calloc ( 555 , 16 ) --> 0xF00DBEAF num: MMM
+calloc ( 5 , 256 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
+calloc ( 1 , 10240 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 '
 
     # lmdbg-leaks + prog4.c
     lmdbg-leaks "$logname" |
-    unify_address | skip_info | skip_useless_addr | sort |
+    unify_address | skip_info | skip_all | sort |
     cmp "prog4.c: lmdbg-leaks + calloc" \
-'calloc ( 1 , 10240 ) --> 0xF00DBEAF num: 4
-realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: 3
+'calloc ( 1 , 10240 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
 '
 fi
 
@@ -704,32 +918,57 @@ if test "$with_posix_memalign" = 1; then
     # lmdbg-run + prog5.c
     logname="$OBJDIR"/_log
 
-    lmdbg-run -o "$logname" "$execname5"
+    lmdbg-run -o "$logname" "$execname5" || true
 
     unify_address "$logname" | skip_info |
-    skip_useless_addr |
+    skip_all | head -5 |
     cmp "prog5.c: lmdbg-run -o" \
-'posix_memalign ( 16 , 200 ) --> 0xF00DBEAF num: 1
-posix_memalign ( 8 , 256 ) --> 0xF00DBEAF num: 2
-realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: 3
-posix_memalign ( 256 , 10240 ) --> 0xF00DBEAF num: 4
-free ( 0xF00DBEAF ) num: 5
+'posix_memalign ( 16 , 200 ) --> 0xF00DBEAF num: MMM
+posix_memalign ( 8 , 256 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
+posix_memalign ( 256 , 10240 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
 '
 
     # lmdbg-leaks + prog5.c
     lmdbg-leaks "$logname" | lmdbg-sym -p |
     unify_paths | hide_line_numbers |
-    unify_address | skip_info | skip_useless_addr |
+    unify_address | skip_info | skip_all |
     hide_foreign_code | sort |
     cmp "prog5.c: lmdbg-leaks + lmdbg-sym" \
-' 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	log_stacktrace
- 0xF00DBEAF	lmdbg.c:NNN	posix_memalign
- 0xF00DBEAF	lmdbg.c:NNN	realloc
+' 0xF00DBEAF	prog5.c:NNN	main
  0xF00DBEAF	prog5.c:NNN	main
- 0xF00DBEAF	prog5.c:NNN	main
-posix_memalign ( 256 , 10240 ) --> 0xF00DBEAF num: 4
-realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: 3
+posix_memalign ( 256 , 10240 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
+'
+fi
+
+if test "$with_aligned_alloc" = 1; then
+    # lmdbg-run + prog9.c
+    logname="$OBJDIR"/_log
+
+    lmdbg-run -o "$logname" "$execname9" || true
+
+    unify_address "$logname" | skip_info |
+    skip_all | head -5 |
+    cmp "prog9.c: lmdbg-run -o" \
+'aligned_alloc ( 16 , 200 ) --> 0xF00DBEAF num: MMM
+aligned_alloc ( 8 , 256 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
+aligned_alloc ( 256 , 10240 ) --> 0xF00DBEAF num: MMM
+free ( 0xF00DBEAF ) num: MMM
+'
+
+    # lmdbg-leaks + prog9.c
+    lmdbg-leaks "$logname" | lmdbg-sym -p |
+    unify_paths | hide_line_numbers |
+    unify_address | skip_info | skip_all |
+    hide_foreign_code | sort |
+    cmp "prog9.c: lmdbg-leaks + lmdbg-sym" \
+' 0xF00DBEAF	prog9.c:NNN	main
+ 0xF00DBEAF	prog9.c:NNN	main
+aligned_alloc ( 256 , 10240 ) --> 0xF00DBEAF num: MMM
+realloc ( 0xF00DBEAF , 1024 ) --> 0xF00DBEAF num: MMM
 '
 fi
 
@@ -745,12 +984,13 @@ ctrl2norm (){
 
 test_fn="$OBJDIR/_tst"
 
-lmdbg-m2s ./input2.txt | ctrl2norm |
+$LMDBG_M2S_DIR/lmdbg-m2s ./input2.txt | ctrl2norm |
 cmp "lmdbg-m2s:" \
 'info lalala \{031}
 malloc ( 123 ) -> 0x1234 \{031}\{034}0x234\{034}0x456
 calloc ( 16 , 124 ) -> 0x1235 \{031}\{034}0x235\{034}0x457\{034}0x678
 memalign ( 16 , 123 ) -> 0x1235000 \{031}\{034}0x1\{034}0x2\{034}0x3
+aligned_alloc ( 8 , 124 ) -> 0x123E000 \{031}\{034}0x1\{034}0x2\{034}0x8
 realloc ( 0x1235000 , 12300 ) -> 0x2236000 \{031}\{034}0x2\{034}0x3\{034}0x4
 posix_memalign ( 16 , 123 ) -> 0x3235000 \{031}\{034}0x1\{033}foo\{034}0x2\{033}bar\{032}baz\{034}0x3\{033}foobar
 stacktrace peak: 123 max: 234 allocs: 456 \{031}\{034}0x111\{034}0x222\{034}0x333
@@ -765,7 +1005,7 @@ ctrl2norm (){
 	print}' "$@"
 }
 
-lmdbg-m2s ./input2.txt | lmdbg-s2m > $test_fn.tmp
+$LMDBG_M2S_DIR/lmdbg-m2s ./input2.txt | $LMDBG_S2M_DIR/lmdbg-s2m > $test_fn.tmp
 printf 'lmdbg-s2m:... ' 1>&2
 if diff ./input2.txt "$test_fn.tmp" > "$test_fn.tmp2"; then
     echo ok
@@ -776,7 +1016,7 @@ else
 fi
 
 # lmdbg-stat: malloc
-lmdbg-stat ./input3.txt | lmdbg-m2s | sort | lmdbg-s2m |
+lmdbg-stat ./input3.txt | $LMDBG_M2S_DIR/lmdbg-m2s | sort | $LMDBG_S2M_DIR/lmdbg-s2m |
 cmp "lmdbg-stat (input3.txt):" \
 'info stat total_allocs: 4
 info stat total_free_cnt: 2
@@ -797,12 +1037,12 @@ stacktrace peak: 120 max: 70 allocs: 3 leaks: 50
 
 stat_fn="$OBJDIR/_stat"
 
-lmdbg-stat ./input1.txt | lmdbg-m2s | sort | lmdbg-s2m | tee "$stat_fn" |
+lmdbg-stat ./input1.txt | $LMDBG_M2S_DIR/lmdbg-m2s | sort | $LMDBG_S2M_DIR/lmdbg-s2m | tee "$stat_fn" |
 cmp "lmdbg-stat (input1.txt):" \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
+info stat total_leaks: 933
 stacktrace peak: 130 max: 130 allocs: 1 leaks: 130
  0x2
  0x3
@@ -811,10 +1051,6 @@ stacktrace peak: 200 max: 200 allocs: 1
 stacktrace peak: 223 max: 123 allocs: 2 leaks: 123
  0x1
  0x2
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
- 0x3
- 0x4
- 0x5
 stacktrace peak: 248 max: 248 allocs: 1
  0x2
  0x3
@@ -823,6 +1059,10 @@ stacktrace peak: 300 max: 300 allocs: 1
  0x5
 stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
  0x6
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
+ 0x3
+ 0x4
+ 0x5
 '
 
 # lmdbg-grep
@@ -927,6 +1167,10 @@ memalign ( 16 , 123 ) -> 0x1235000
  0x1
  0x2
  0x3
+aligned_alloc ( 8 , 124 ) -> 0x123E000
+ 0x1
+ 0x2
+ 0x8
 realloc ( 0x1235000 , 12300 ) -> 0x2236000
  0x2
  0x3
@@ -951,39 +1195,35 @@ stacktrace peak: 123 max: 234 allocs: 456
 lmdbg-grep 'max > 100 && max < 200' "$stat_fn" |
 cmp 'lmdbg-grep + max' \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
+info stat total_leaks: 933
 stacktrace peak: 130 max: 130 allocs: 1 leaks: 130
  0x2
  0x3
 stacktrace peak: 223 max: 123 allocs: 2 leaks: 123
  0x1
  0x2
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
+stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
+ 0x6
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
  0x3
  0x4
  0x5
-stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
- 0x6
 '
 
 # lmdbg-grep
 lmdbg-grep -v 'peak < 200' "$stat_fn" |
 cmp 'lmdbg-grep -v + peak' \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
+info stat total_leaks: 933
 stacktrace peak: 200 max: 200 allocs: 1
  0x7
 stacktrace peak: 223 max: 123 allocs: 2 leaks: 123
  0x1
  0x2
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
- 0x3
- 0x4
- 0x5
 stacktrace peak: 248 max: 248 allocs: 1
  0x2
  0x3
@@ -992,15 +1232,23 @@ stacktrace peak: 300 max: 300 allocs: 1
  0x5
 stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
  0x6
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
+ 0x3
+ 0x4
+ 0x5
 '
 
 # lmdbg-sort
 lmdbg-sort -f peak < "$stat_fn" |
 cmp 'lmdbg-sort -f peak' \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
+info stat total_leaks: 933
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
+ 0x3
+ 0x4
+ 0x5
 stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
  0x6
 stacktrace peak: 300 max: 300 allocs: 1
@@ -1009,10 +1257,6 @@ stacktrace peak: 248 max: 248 allocs: 1
  0x2
  0x3
  0x4
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
- 0x3
- 0x4
- 0x5
 stacktrace peak: 223 max: 123 allocs: 2 leaks: 123
  0x1
  0x2
@@ -1026,15 +1270,15 @@ stacktrace peak: 130 max: 130 allocs: 1 leaks: 130
 lmdbg-sort -f leaks < "$stat_fn" |
 cmp 'lmdbg-sort -f leaks' \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
-stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
- 0x6
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
+info stat total_leaks: 933
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
  0x3
  0x4
  0x5
+stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
+ 0x6
 stacktrace peak: 130 max: 130 allocs: 1 leaks: 130
  0x2
  0x3
@@ -1054,9 +1298,9 @@ stacktrace peak: 300 max: 300 allocs: 1
 lmdbg-sort -f max < "$stat_fn" |
 cmp 'lmdbg-sort -f max' \
 'info lalala
-info stat total_allocs: 13
+info stat total_allocs: 14
 info stat total_free_cnt: 2
-info stat total_leaks: 793
+info stat total_leaks: 933
 stacktrace peak: 300 max: 300 allocs: 1
  0x5
 stacktrace peak: 248 max: 248 allocs: 1
@@ -1067,16 +1311,16 @@ stacktrace peak: 200 max: 200 allocs: 1
  0x7
 stacktrace peak: 310 max: 180 allocs: 5 leaks: 310
  0x6
+stacktrace peak: 370 max: 140 allocs: 3 leaks: 370
+ 0x3
+ 0x4
+ 0x5
 stacktrace peak: 130 max: 130 allocs: 1 leaks: 130
  0x2
  0x3
 stacktrace peak: 223 max: 123 allocs: 2 leaks: 123
  0x1
  0x2
-stacktrace peak: 230 max: 120 allocs: 2 leaks: 230
- 0x3
- 0x4
- 0x5
 '
 
 lmdbg-sort -f leaks input5.txt |
@@ -1228,6 +1472,10 @@ memalign ( 16 , 110 ) --> 0xXYZ
  0x5
 realloc ( 0xXYZ , 180 ) --> 0xXYZ
  0x6
+aligned_alloc ( 8 , 140 ) --> 0xXYZ
+ 0x3
+ 0x4
+ 0x5
 '
 
 lmdbg-strip -al input4.txt |
@@ -1288,6 +1536,55 @@ stacktrace peak: 90 max: 90 allocs: 1
  0x8048517
 '
 
+# lmdbg-strip
+lmdbg-head input1.txt |
+cmp 'lmdbg-head' \
+'info lalala
+malloc ( 123 ) --> 0x1234
+ 0x1
+ 0x2
+calloc ( 2 , 124 ) --> 0x1235
+ 0x2
+ 0x3
+ 0x4
+memalign ( 16 , 120 ) --> 0x1236
+ 0x3
+ 0x4
+ 0x5
+free ( 0x1235 )
+ 0x1
+posix_memalign ( 16 , 130 ) --> 0x1237
+ 0x2
+ 0x3
+malloc ( 100 ) --> 0x1238
+ 0x1
+ 0x2
+malloc ( 200 ) --> 0x1239
+ 0x7
+realloc ( 0x1238 , 300 ) --> 0x123A
+ 0x5
+free ( 0x1239 )
+ 0x1
+realloc ( 0x123A , 110 ) --> 0x123B
+ 0x6
+'
+
+lmdbg-head -n3 input1.txt |
+cmp 'lmdbg-head -n3' \
+'info lalala
+malloc ( 123 ) --> 0x1234
+ 0x1
+ 0x2
+calloc ( 2 , 124 ) --> 0x1235
+ 0x2
+ 0x3
+ 0x4
+memalign ( 16 , 120 ) --> 0x1236
+ 0x3
+ 0x4
+ 0x5
+'
+
 #
 if test -f "$errfn"; then
     exit 1
